package org.artifactory.metrics.providers.features;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.retention.ArtifactRetentionAddon;
import org.artifactory.addon.retention.ArtifactRetentionStats;
import org.artifactory.api.callhome.FeatureGroup;
import org.artifactory.util.UnsupportedByLicenseException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Objects;

/**
 * @author Mor Merhav
 */
@Component
public class ArtifactRetentionFeature implements CallHomeFeature {

    private static final String MAIN_FEATURE_NAME = "Artifactory On Ice";
    private static final String IS_COLD_INSTANCE = "Is_Cold_Instance";
    private static final String TOTAL_POLICIES_NO = "Total_Policies";
    private static final String LAST_DAY_ARCHIVE_POLICIES_RUNS = "Last_Day_Archive_Policies_Runs";
    private static final String LAST_DAY_ARCHIVED_ARTIFACTS = "Last_Day_Archived_Artifacts";
    private static final String LAST_DAY_ARCHIVED_ARTIFACTS_SIZE = "Last_Day_Archived_Artifacts_Size";
    private static final String LAST_DAY_ARCHIVED_ARTIFACTS_FAILED = "Last_Day_Archived_Artifacts_Failed";
    private static final String LAST_DAY_ARCHIVED_ARTIFACTS_FAILED_SIZE = "Last_Day_Archived_Artifacts_Failed_Size";
    private static final String LAST_DAY_RESTORE_RUNS = "Last_Day_Restore_Runs";
    private static final String LAST_DAY_RESTORE_ARTIFACTS = "Last_Day_Restored_Artifacts";
    private static final String LAST_DAY_RESTORE_ARTIFACTS_SIZE = "Last_Day_Restored_Artifacts_Size";
    private static final String LAST_DAY_RESTORE_ARTIFACTS_FAILED = "Last_Day_Restored_Artifacts_Failed";
    private static final String LAST_DAY_RESTORE_ARTIFACTS_FAILED_SIZE = "Last_Day_Restored_Artifacts_Failed_Size";

    private final AddonsManager addonsManager;

    @Autowired
    public ArtifactRetentionFeature(AddonsManager addonsManager) {
        this.addonsManager = addonsManager;
    }

    @Override
    public FeatureGroup getFeature() {
        ArtifactRetentionAddon artifactRetentionAddon = addonsManager.addonByType(ArtifactRetentionAddon.class);

        if (!artifactRetentionAddon.isRetentionEnabled()) {
            return null;
        }

        boolean coldInstance = artifactRetentionAddon.isColdInstance();

        FeatureGroup feature = new FeatureGroup(MAIN_FEATURE_NAME);
        feature.addFeatureAttribute(IS_COLD_INSTANCE, coldInstance);

        if (coldInstance) {
            return feature;
        }

        ArtifactRetentionStats artifactRetentionStats;
        try {
            artifactRetentionStats = artifactRetentionAddon.getArtifactRetentionStats();
        } catch (UnsupportedByLicenseException ignored) {
            return null;
        }

        if (Objects.isNull(artifactRetentionStats)) {
            return null;
        }

        feature.addFeatureAttribute(TOTAL_POLICIES_NO, artifactRetentionStats.getPoliciesCount());
        feature.addFeatureAttribute(LAST_DAY_ARCHIVE_POLICIES_RUNS, artifactRetentionStats.getArchiveRunCount());
        feature.addFeatureAttribute(LAST_DAY_ARCHIVED_ARTIFACTS, artifactRetentionStats.getArchiveSuccessCount());
        feature.addFeatureAttribute(LAST_DAY_ARCHIVED_ARTIFACTS_SIZE, artifactRetentionStats.getArchiveSuccessSize());
        feature.addFeatureAttribute(LAST_DAY_ARCHIVED_ARTIFACTS_FAILED,
                artifactRetentionStats.getArchiveFailureCount());
        feature.addFeatureAttribute(LAST_DAY_ARCHIVED_ARTIFACTS_FAILED_SIZE,
                artifactRetentionStats.getArchiveFailureSize());
        feature.addFeatureAttribute(LAST_DAY_RESTORE_RUNS, artifactRetentionStats.getRestoreRunCount());
        feature.addFeatureAttribute(LAST_DAY_RESTORE_ARTIFACTS, artifactRetentionStats.getRestoreSuccessCount());
        feature.addFeatureAttribute(LAST_DAY_RESTORE_ARTIFACTS_SIZE, artifactRetentionStats.getRestoreSuccessSize());
        feature.addFeatureAttribute(LAST_DAY_RESTORE_ARTIFACTS_FAILED, artifactRetentionStats.getRestoreFailureCount());
        feature.addFeatureAttribute(LAST_DAY_RESTORE_ARTIFACTS_FAILED_SIZE,
                artifactRetentionStats.getRestoreFailureSize());

        return feature;
    }
}
