package org.artifactory.schedule;

import org.mockito.Mockito;
import org.quartz.JobDetail;
import org.quartz.JobKey;
import org.quartz.SimpleScheduleBuilder;
import org.quartz.Trigger;
import org.quartz.TriggerKey;
import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.Date;

/**
 * @author Alexei Vainshtein
 */
public class QuartzTaskTest {

    private QuartzTask quartzTask;
    private TriggerKey key = new TriggerKey("test", "artifactory");

    @BeforeMethod
    private void prepareQuartzTask() {
        Trigger trigger = Mockito.mock(Trigger.class);
        Mockito.when(trigger.getKey()).thenReturn(key);
        JobDetail jobDetail = Mockito.mock(JobDetail.class);
        Mockito.when(jobDetail.getKey()).thenReturn(new JobKey("test"));
        quartzTask = Mockito.spy(new QuartzTask(HaQuartzTaskTest.Job1.class, trigger, jobDetail));
    }

    @Test
    public void testShouldRescheduleTask() {
        quartzTask.setShouldRescheduleTask(true);
        Assert.assertTrue(quartzTask.shouldRescheduleTask());
        quartzTask.setShouldRescheduleTask(false);
        Assert.assertFalse(quartzTask.shouldRescheduleTask());
    }

    @Test
    public void testRescheduleTask() {
        Date currentTime = new Date(System.currentTimeMillis());
        Mockito.doNothing().when(quartzTask).cancelTask();
        Mockito.doNothing().when(quartzTask).scheduleTask();
        quartzTask.setShouldRescheduleTask(true);
        quartzTask.rescheduleTask(1000);
        Mockito.verify(quartzTask, Mockito.times(1)).cancelTask();
        Mockito.verify(quartzTask, Mockito.times(1)).scheduleTask();
        Trigger trigger = quartzTask.getTrigger();
        Assert.assertEquals(trigger.getKey(), key);
        Assert.assertTrue(trigger.getScheduleBuilder() instanceof SimpleScheduleBuilder);
        Date startTime = trigger.getStartTime();
        Assert.assertTrue(startTime.getTime() >= currentTime.getTime() + 1000);
        Assert.assertFalse(quartzTask.shouldRescheduleTask());
    }
}