package org.artifactory.servicetrust;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.federated.FederatedRepoAddon;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.spring.InternalArtifactoryContext;
import org.artifactory.test.ArtifactoryHomeBoundTest;


import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.security.core.context.SecurityContextHolder;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.util.Optional;

import static org.mockito.Mockito.when;

public class ServiceTrustServiceTest extends ArtifactoryHomeBoundTest {

    @Mock
    private InternalArtifactoryContext context;

    @Mock
    private AddonsManager addonsManager;

    @Mock
    private FederatedRepoAddon federatedRepoAddon;

    private AutoCloseable closeableMock;

    @BeforeTest
    private void setup() {
        closeableMock = MockitoAnnotations.openMocks(this);
        ArtifactoryContextThreadBinder.bind(context);
        when(addonsManager.addonByType(FederatedRepoAddon.class)).thenReturn(federatedRepoAddon);
        when(context.beanForType(AddonsManager.class)).thenReturn(addonsManager);
    }

    @AfterMethod
    private void cleanup() throws Exception {
        ArtifactoryContextThreadBinder.unbind();
        SecurityContextHolder.clearContext();
        closeableMock.close();
    }



    @Test(dataProvider = "validUsecaseProvider")
    public void findUsecaseSucessTest(String usecase) {
        ServiceTrustService service = new ServiceTrustServiceImpl();
        Optional<ServiceTrustUseCase> useCase = service.findUseCase(usecase);
        Assert.assertNotNull(useCase);
    }

    @Test(dataProvider = "invalidUsecaseProvider",
            expectedExceptions = {ServiceTrustException.class})
    public void findUsecaseFailTest(String usecase) {
        ServiceTrustService service = new ServiceTrustServiceImpl();
        service.findUseCase(usecase);

    }

    @DataProvider
    private static Object[][] validUsecaseProvider() {
        return new Object[][] {
                {"federated-repo/fedrepo-1"},
                {"federated-repo/*"},
                {"internal:federated-repo/*"},
        };
    }

    @DataProvider
    private static Object[][] invalidUsecaseProvider() {
        return new Object[][] {
                {"federated-repo/fed-repo-1 "},
                {"  federated-repo/fed-repo-1"},
                {"federated-repo/  fed-repo-1"},
                {"   federated-repo/*"},
        };
    }


}
