/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.security.providermgr;

import org.artifactory.api.context.ContextHelper;
import org.artifactory.security.LoginHandler;
import org.artifactory.security.props.auth.model.OauthErrorEnum;
import org.artifactory.security.props.auth.model.OauthErrorModel;
import org.artifactory.security.props.auth.model.OauthModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServletResponse;

/**
 * @author Chen Keinan
 */
public class OAuthProviderMgr implements ProviderMgr {
    private static final Logger log = LoggerFactory.getLogger(OAuthProviderMgr.class);

    public OAuthProviderMgr() {
    }

    @Override
    public final OauthModel fetchAndStoreTokenFromProvider(String username) {
        LoginHandler loginHandler = ContextHelper.get().beanForType(LoginHandler.class);
        try {
            return loginHandler.doBasicAuthWithDb(username);
        } catch (Exception e) {
            log.error("Failed to authenticate with basic authentication: {}", e.getMessage());
            log.debug("Failed to authenticate with basic authentication: {}", e.getMessage(), e);
        }
        OauthModel oauthModel =
                createErrorModel(HttpServletResponse.SC_UNAUTHORIZED, OauthErrorEnum.BAD_CREDENTIAL);
        log.debug("Artifactory basic authentication failed.");
        return oauthModel;
    }

    protected OauthModel createErrorModel(int statusCode, OauthErrorEnum errorEnum) {
        return new OauthErrorModel(statusCode, errorEnum);
    }
}
