package org.artifactory.opentracing;

import org.artifactory.api.events.EventLogHintManager;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;

import static org.artifactory.api.events.EventHintType.FederatedReplication;
import static org.artifactory.mirror.MirrorConstants.MIRROR_MARKER;

class TraceableRunnableDecoratorTest {

    @Test
    void testMirrorMarkerIsSetAndRemovedOnEachRunnable() {
        ArtifactoryTracer artifactoryTracer = Mockito.mock(ArtifactoryTracer.class);
        ServerTracingSpan currentSpan = Mockito.mock(ServerTracingSpan.class);
        Mockito.when(artifactoryTracer.getCurrentThreadActiveSpan()).thenReturn(currentSpan);
        TraceableRunnableDecorator runnableDecorator =
                new TraceableRunnableDecorator(assertMirrorMarker, artifactoryTracer, true);

        Assertions.assertNull(EventLogHintManager.getHints());
        runnableDecorator.run();
        Assertions.assertNull(EventLogHintManager.getHints());
    }

    private final Runnable assertMirrorMarker = () -> {
        Assertions.assertNotNull(EventLogHintManager.getHints());
        Assertions
                .assertTrue(EventLogHintManager.getHints().contains(FederatedReplication.getPrefix() + MIRROR_MARKER));
    };
}
