#!/bin/bash

export version=${version:-${VERSION}}
export ARTIFACTORY_OSS_SOURCES_PATH=bintray-artifactory/org/artifactory/oss/jfrog-artifactory-oss/${version}/jfrog-artifactory-oss-${version}-sources.tar.gz
export ARTIFACTORY_OSS_SOURCES_TAR_GZ_FILE=jfrog-artifactory-oss-${version}-sources.tar.gz

function checkoutToTag() {
  local tag
  tag=$(git tag | grep -E "(artifactory|artifactory-oss)/${version}\$")
  git -C ${res_artifactory_third_party_bitbucket_resourcePath} checkout "${tag}"
}

function checkoutPreviousBranch() {
    git checkout -
}

function removePrivateFiles() {
  filesToDelete="artifactory-oss/.git*
artifactory-oss/Jenkinsfile
artifactory-oss/jenkins
artifactory-oss/web/angular-web/src/main/webapp/yarn.lock
artifactory-oss/jfrog-build-artifactory-oss.groovy
artifactory-oss/JenkinsStork.groovy
artifactory-oss/nightly-release.groovy"

  for file in ${filesToDelete}; do
    echo "Deleting ${file}"
    rm -rf ${file}
  done
}

function createOssSourcesPackage() {
  mkdir oss-sources
  mv artifactory-oss/ oss-sources/jfrog-artifactory-${version}
  pushd oss-sources
  tar -zcf ${ARTIFACTORY_OSS_SOURCES_TAR_GZ_FILE} jfrog-artifactory-${version}/
  popd
}

function uploadOssSources() {
  pushd oss-sources
  jfrog rt u ${ARTIFACTORY_OSS_SOURCES_TAR_GZ_FILE} ${ARTIFACTORY_OSS_SOURCES_PATH}
  popd
}

function downloadOssSources() {
    local target_dir=oss-downloaded-sources
    mkdir -p ${target_dir}
    pushd ${target_dir}
    jfrog rt dl --flat ${ARTIFACTORY_OSS_SOURCES_PATH} ${ARTIFACTORY_OSS_SOURCES_TAR_GZ_FILE}
    tar -xzf ${ARTIFACTORY_OSS_SOURCES_TAR_GZ_FILE}
    popd
}

function generateMavenConfigFile() {
    pushd oss-downloaded-sources
    cat > config.yaml << EOF
        version: 1
        type: maven
        resolver:
          serverId: releases_jfrog_io
          releaseRepo: jcenter
EOF
    popd
}
