package org.artifactory.storage.db.build.itest.dao;

import org.artifactory.api.build.ModuleArtifact;
import org.artifactory.api.build.ModuleDependency;
import org.artifactory.storage.db.binstore.dao.BinariesDao;
import org.artifactory.storage.db.build.dao.BuildArtifactsDao;
import org.artifactory.storage.db.build.dao.BuildDependenciesDao;
import org.artifactory.storage.db.build.dao.BuildModulesDao;
import org.artifactory.storage.db.build.dao.BuildsDao;
import org.artifactory.storage.db.fs.dao.NodesDao;
import org.artifactory.storage.db.fs.dao.PropertiesDao;
import org.artifactory.storage.db.itest.DbBaseTest;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import java.sql.SQLException;
import java.util.List;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

@Test
public class BuildArtifactsAndDependenciesPathTest extends DbBaseTest {

    @Autowired
    protected BuildModulesDao buildModulesDao;

    @Autowired
    protected BuildsDao buildsDao;

    @Autowired
    protected BuildArtifactsDao buildArtifactsDao;

    @Autowired
    protected BuildDependenciesDao buildDependenciesDao;

    @Autowired
    private BinariesDao binariesDao;

    @Autowired
    private NodesDao nodesDao;

    @Autowired
    private PropertiesDao propertiesDao;

    @BeforeClass
    public void setup() throws SQLException {
        buildArtifactsDao.deleteAllBuildArtifacts();
        buildDependenciesDao.deleteAllBuildDependencies();
        buildModulesDao.deleteAllBuildModules();
        buildsDao.deleteAllBuilds(false);
        importSql("/sql/build-path-matching.sql");
    }

    @AfterClass
    public void cleanup() throws SQLException {
        assertEquals(buildArtifactsDao.deleteAllBuildArtifacts(), 2);
        assertEquals(buildDependenciesDao.deleteAllBuildDependencies(), 2);
        assertEquals(buildModulesDao.deleteAllBuildModules(), 2);
        assertEquals(buildsDao.deleteAllBuilds(false), 3);
        for (int i = 1000; i <= 1001; i++) {
            assertTrue(propertiesDao.deleteNodeProperties(i) > 1);
            assertTrue(nodesDao.delete(i));
        }
        binariesDao.deleteEntry("sha1-01");
    }

    @Test
    public void whenNoPromotionThenTakeFirstSuitableArtifactNode() throws SQLException {
        List<ModuleArtifact> artifacts = buildsDao.getModuleArtifact("build-without-promotions",
                "1", "1", "module-1", "artifactory-build-info", true);
        assertEquals(artifacts.size(), 1);
        String repoKey = artifacts.get(0).getRepoKey();
        assertTrue(repoKey.equals("repo-01") || repoKey.equals("repo-02"));
    }

    @Test
    public void whenNoPromotionThenTakeFirstSuitableDependencyNode() throws SQLException {
        List<ModuleDependency> artifacts = buildsDao.getModuleDependency("build-without-promotions",
                "1", "1", "artifactory-build-info", "module-1");
        assertEquals(artifacts.size(), 1);
        String repoKey = artifacts.get(0).getRepoKey();
        assertTrue(repoKey.equals("repo-01") || repoKey.equals("repo-02"));
    }

    @Test
    public void whenPromotionExistsThenTakeArtifactNodeFromPromotionRepo() throws SQLException {
        List<ModuleArtifact> artifacts = buildsDao.getModuleArtifact("build-with-promotions",
                "1", "1", "module-2", "artifactory-build-info", true);
        assertEquals(artifacts.size(), 1);
        assertEquals(artifacts.get(0).getRepoKey(), "repo-02");
    }

    @Test
    public void whenPromotionExistsThenTakeDependencyNodeFromPromotionRepo() throws SQLException {
        List<ModuleDependency> artifacts = buildsDao.getModuleDependency("build-with-promotions",
                "1", "1", "artifactory-build-info", "module-2");
        assertEquals(artifacts.size(), 1);
        assertEquals(artifacts.get(0).getRepoKey(), "repo-02");
    }

}

