/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.utils;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;
import org.artifactory.descriptor.repo.RepoType;

import java.util.EnumSet;
import java.util.Set;

/**
 * @author Nadav Yogev
 */
@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class ReindexUtils {

    private static final Set<RepoType> LOCAL_REPO_REINDEX = EnumSet.of(RepoType.Gems, RepoType.Npm, RepoType.Bower,
            RepoType.CocoaPods, RepoType.NuGet, RepoType.Debian, RepoType.Opkg, RepoType.YUM, RepoType.Pypi,
            RepoType.Composer, RepoType.Chef, RepoType.Puppet, RepoType.Helm, RepoType.CRAN, RepoType.Conda,
            RepoType.Conan, RepoType.Alpine, RepoType.Cargo, RepoType.Pub, RepoType.Terraform, RepoType.Swift);
    private static final Set<RepoType> REMOTE_REPO_REINDEX = EnumSet.of(RepoType.Bower, RepoType.NuGet, RepoType.Cargo);
    private static final Set<RepoType> VIRTUAL_REPO_REINDEX =
            EnumSet.of(RepoType.Gems, RepoType.Npm, RepoType.Bower, RepoType.NuGet, RepoType.Debian);

    public static boolean hasLocalReindex(RepoType type) {
        return LOCAL_REPO_REINDEX.contains(type);
    }

    public static boolean hasRemoteReindex(RepoType type) {
        return REMOTE_REPO_REINDEX.contains(type);
    }

    public static boolean hasVirtualReindex(RepoType type) {
        return VIRTUAL_REPO_REINDEX.contains(type);
    }

}
